# Generated by Django 2.1.9 on 2019-06-19 12:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("mds", "0027_eventrecord_first_saved_at")]

    operations = [
        migrations.AddField(
            model_name="provider", name="operator", field=models.BooleanField(null=True)
        )
    ]
