# Generated by Django 2.2 on 2019-05-17 08:38

from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [("mds", "0023_relabel_api_fields")]

    operations = [
        migrations.AlterField(
            model_name="provider",
            name="device_category",
            field=mds.models.UnboundedCharField(
                blank=True,
                choices=[
                    ("bicycle", "Bicycle"),
                    ("scooter", "Scooter"),
                    ("car", "Car"),
                ],
            ),
        )
    ]
