# Generated by Django 2.1.5 on 2019-03-15 15:52

from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0013_provider_last_start_time_polled'),
    ]

    operations = [
        migrations.AlterField(
            model_name='area',
            name='label',
            field=mds.models.UnboundedCharField(blank=True, db_index=True, default=''),
        ),
        migrations.AlterField(
            model_name='device',
            name='manufacturer',
            field=mds.models.UnboundedCharField(default=''),
        ),
        migrations.AlterField(
            model_name='device',
            name='model',
            field=mds.models.UnboundedCharField(default=''),
        ),
        migrations.AlterField(
            model_name='eventrecord',
            name='event_type',
            field=mds.models.UnboundedCharField(choices=[('service_start', 'Service start'), ('trip_end', 'Trip end'), ('rebalance_drop_off', 'Rebalance drop off'), ('maintenance_drop_off', 'Maintenance drop off'), ('cancel_reservation', 'Cancel reservation'), ('reserve', 'Reserve'), ('trip_start', 'Trip start'), ('trip_enter', 'Trip enter'), ('trip_leave', 'Trip leave'), ('register', 'Register'), ('low_battery', 'Low battery'), ('maintenance', 'Maintenance'), ('service_end', 'Service end'), ('rebalance_pick_up', 'Rebalance pick up'), ('maintenance_pick_up', 'Maintenance pick up'), ('deregister', 'Deregister'), ('telemetry', 'Received telemetry'), ('battery_charged', 'Battery charged')]),
        ),
        migrations.AlterField(
            model_name='polygon',
            name='label',
            field=mds.models.UnboundedCharField(blank=True, db_index=True, default=''),
        ),
        migrations.AlterField(
            model_name='provider',
            name='name',
            field=mds.models.UnboundedCharField(default=''),
        ),
    ]
