# Generated by Django 2.1.5 on 2019-03-20 15:47

import django.contrib.postgres.fields.jsonb
from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0015_null_dn_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='provider',
            name='agency_api_configuration',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=mds.models.agency_api_configuration_default, verbose_name='Agency API Configuration'),
        ),
        migrations.AlterField(
            model_name='provider',
            name='agency_api_authentication',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=mds.models.agency_api_authentication_default, verbose_name='Agency API Authentication'),
        ),
        migrations.AlterField(
            model_name='provider',
            name='api_authentication',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=mds.models.provider_api_authentication_default, verbose_name='Provider API Authentication'),
        ),
        migrations.AlterField(
            model_name='provider',
            name='api_configuration',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=mds.models.provider_api_configuration_default, verbose_name='Provider API Configuration'),
        ),
        migrations.AlterField(
            model_name='provider',
            name='base_api_url',
            field=mds.models.UnboundedCharField(default='', verbose_name='Base Provider API URL'),
        ),
    ]
