# Generated by Django 2.1.10 on 2019-07-23 17:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("mds", "0034_post_default_device_saved_at")]

    operations = [
        migrations.AddField(
            model_name="provider",
            name="last_event_time_polled",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="Last event time polled (start_time)",
            ),
        ),
        migrations.AddField(
            model_name="provider",
            name="last_recorded_polled",
            field=models.DateTimeField(
                blank=True,
                null=True,
                verbose_name="Last recorded polled (start_recorded)",
            ),
        ),
        migrations.AddField(
            model_name="provider",
            name="last_skip_polled",
            field=models.PositiveIntegerField(
                blank=True, null=True, verbose_name="Last skip polled (total_events)"
            ),
        ),
    ]
