# Generated by Django 2.1.8 on 2019-05-28 14:52

import django.contrib.gis.db.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0025_remove_provider_device_category'),
    ]

    operations = [
        migrations.AlterField(
            model_name='polygon',
            name='geom',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326),
        ),
    ]
