# Generated by Django 2.1.5 on 2019-03-14 15:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0012_update_to_mds_0_3_0'),
    ]

    operations = [
        migrations.AddField(
            model_name='provider',
            name='last_start_time_polled',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
