# Generated by Django 2.1.3 on 2019-06-18 13:52

import django.contrib.postgres.functions
from django.db import migrations, models


class Migration(migrations.Migration):
    # concurrent index creation cannot be made inside a transaction
    atomic = False

    dependencies = [("mds", "0030_post_provider_operator_default")]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.AlterField(
                    model_name="eventrecord",
                    name="saved_at",
                    field=models.DateTimeField(
                        db_index=True,
                        default=django.contrib.postgres.functions.TransactionNow,
                    ),
                )
            ],
            database_operations=[
                migrations.RunSQL(
                    sql="""
                    CREATE INDEX CONCURRENTLY "mds_eventrecord_saved_at_f421e5fd"
                    ON "mds_eventrecord" ("saved_at");
                    """
                )
            ],
        )
    ]
