# Generated by Django 2.2 on 2019-07-03 11:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("mds", "0033_post_fill_device_saved_at")]

    operations = [
        migrations.AlterField(
            model_name="device",
            name="saved_at",
            field=models.DateTimeField(db_index=True, auto_now=True),
        )
    ]
