# Generated by Django 2.1.5 on 2019-01-22 13:00

from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0004_eventrecord_point'),
    ]

    operations = [
        migrations.AlterField(
            model_name='device',
            name='dn_status',
            field=mds.models.UnboundedCharField(choices=[('available', 'Available'), ('reserved', 'Reserved'), ('unavailable', 'Unavailable'), ('removed', 'Removed'), ('unknown', 'Unknown')], default='unavailable'),
        ),
    ]
