# Generated by Django 2.1.10 on 2019-07-23 17:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("mds", "0036_pre_save_provider_cursors")]

    operations = [
        migrations.RemoveField(model_name="provider", name="last_start_time_polled")
    ]

