# Generated by Django 2.2.2 on 2019-06-07 14:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("mds", "0026_alter_polygon_to_multipolygon")]

    operations = [
        migrations.AddField(
            model_name="eventrecord",
            name="first_saved_at",
            field=models.DateTimeField(blank=True, null=True),
        )
    ]
